@@ -34,7 +34,6 @@ in the intermediate representation under the key "foo".
 # Python modules
 from itertools import chain
 import numpy
-from six.moves import range
 
 # Import UFL
 import ufl
@@ -46,8 +45,6 @@ from ffc.fiatinterface import create_element, reference_cell
 from ffc.fiatinterface import EnrichedElement, HDivTrace, MixedElement, SpaceOfReals, QuadratureElement
 from ffc.classname import make_classname, make_integral_classname
 
-from six import itervalues
-
 # List of supported integral types
 ufc_integral_types = ("cell",
                       "exterior_facet",
@@ -844,7 +841,7 @@ def _tabulate_facet_dofs(element, cell):
     num_facets = cell.num_facets()
 
     # Make list of dofs
-    facet_dofs = list(itervalues(element.entity_closure_dofs()[D-1]))
+    facet_dofs = list(element.entity_closure_dofs()[D-1].values())
 
     assert num_facets == len(facet_dofs)
 
