@@ -2,7 +2,7 @@
 """This module contains utility functions for some code shared between
 quadrature and tensor representation."""
 
-# Copyright (C) 2012-2016 Marie Rognes
+# Copyright (C) 2012-2017 Marie Rognes
 #
 # This file is part of FFC.
 #
@@ -19,30 +19,29 @@ quadrature and tensor representation."""
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Martin Sandve Alnæs 2013-2016
+# Modified by Martin Sandve Alnæs 2013-2017
 # Modified by Anders Logg 2014
 
 import numpy
 
-from ufl.measure import integral_type_to_measure_name
+from ufl.measure import integral_type_to_measure_name, point_integral_types, facet_integral_types, custom_integral_types
 from ufl.cell import cellname2facetname
-from ufl import custom_integral_types
 
 from ffc.log import error
-from ffc.cpp import make_integral_classname
 from ffc.fiatinterface import create_element
 from ffc.fiatinterface import create_quadrature
 from ffc.fiatinterface import map_facet_points
 from ffc.fiatinterface import reference_cell_vertices
+from ffc.classname import make_integral_classname
 
 
 def create_quadrature_points_and_weights(integral_type, cell, degree, rule):
     "Create quadrature rule and return points and weights."
     if integral_type == "cell":
         (points, weights) = create_quadrature(cell.cellname(), degree, rule)
-    elif integral_type == "exterior_facet" or integral_type == "interior_facet":
+    elif integral_type in facet_integral_types:
         (points, weights) = create_quadrature(cellname2facetname[cell.cellname()], degree, rule)
-    elif integral_type == "vertex":
+    elif integral_type in point_integral_types:
         (points, weights) = create_quadrature("vertex", degree, rule)
     elif integral_type in custom_integral_types:
         (points, weights) = (None, None)
@@ -55,9 +54,9 @@ def integral_type_to_entity_dim(integral_type, tdim):
     "Given integral_type and domain tdim, return the tdim of the integration entity."
     if integral_type == "cell":
         entity_dim = tdim
-    elif (integral_type == "exterior_facet" or integral_type == "interior_facet"):
+    elif integral_type in facet_integral_types:
         entity_dim = tdim - 1
-    elif integral_type == "vertex":
+    elif integral_type in point_integral_types:
         entity_dim = 0
     elif integral_type in custom_integral_types:
         entity_dim = tdim
@@ -76,7 +75,7 @@ def map_integral_points(points, integral_type, cell, entity):
         return numpy.asarray(points)
     elif entity_dim == tdim - 1:
         assert points.shape[1] == tdim - 1
-        return numpy.asarray(map_facet_points(points, entity))
+        return numpy.asarray(map_facet_points(points, entity, cell.cellname()))
     elif entity_dim == 0:
         return numpy.asarray([reference_cell_vertices(cell.cellname())[entity]])
     else:
@@ -224,5 +223,5 @@ def initialize_integral_code(ir, prefix, parameters):
     code["initializer_list"] = ""
     code["destructor"] = ""
     code["enabled_coefficients"] = generate_enabled_coefficients(ir["enabled_coefficients"])
-    # code["additional_includes_set"] = set() #ir["additional_includes_set"]
+    code["additional_includes_set"] = set()  # FIXME: Get this out of code[]
     return code
