@@ -2,10 +2,6 @@
 """
 Tests of manipulations of the ssa form of expressions.
 """
-
-from __future__ import print_function
-
-from six.moves import map
 from ufl import *
 from ufl import product
 from ufl.permutation import compute_indices
